//
// melody sequencer
//

module seq(clk, rst, start,
           modFrequ, modDecay,
           carFrequ, carDecay);
    input clk;
    input rst;
    output start;
    output reg [15:0] modFrequ;
    output reg [15:0] modDecay;
    output reg [15:0] carFrequ;
    output reg [15:0] carDecay;

  reg [27:0] prescaler;
  wire beat;
  reg [5:0] state;
  reg trigger;
  reg [2:0] note;

  assign beat = (prescaler == 28'd80000000 ? 1 : 0);

  always @(posedge clk) begin
    if (rst) begin
      prescaler <= 28'd0;
    end else begin
      if (beat) begin
        prescaler <= 28'd0;
      end else begin
        prescaler <= prescaler + 1;
      end
    end
  end

  always @(posedge clk) begin
    if (rst) begin
      state <= 0;
    end else begin
      if (beat) begin
        if (state != 6'd63) begin
          state <= state + 1;
        end
      end
    end
  end

  // melody ROM
  // map state to trigger and note number
  always @(*) begin
    case (state)
      6'd01:    begin
                  trigger = 1;
                  note = 3;
                end
      6'd02:    begin
                  trigger = 1;
                  note = 5;
                end
      6'd03:    begin
                  trigger = 1;
                  note = 4;
                end
      6'd04:    begin
                  trigger = 1;
                  note = 2;
                end
      6'd07:    begin
                  trigger = 1;
                  note = 3;
                end
      6'd08:    begin
                  trigger = 1;
                  note = 4;
                end
      6'd09:    begin
                  trigger = 1;
                  note = 5;
                end
      6'd10:    begin
                  trigger = 1;
                  note = 3;
                end
      6'd13:    begin
                  trigger = 1;
                  note = 5;
                end
      6'd14:    begin
                  trigger = 1;
                  note = 3;
                end
      6'd15:    begin
                  trigger = 1;
                  note = 4;
                end
      6'd16:    begin
                  trigger = 1;
                  note = 2;
                end
      6'd19:    begin
                  trigger = 1;
                  note = 2;
                end
      6'd20:    begin
                  trigger = 1;
                  note = 4;
                end
      6'd21:    begin
                  trigger = 1;
                  note = 5;
                end
      6'd22:    begin
                  trigger = 1;
                  note = 3;
                end
      6'd28:    begin
                  trigger = 1;
                  note = 1;
                end
      6'd34:    begin
                  trigger = 1;
                  note = 1;
                end
      6'd40:    begin
                  trigger = 1;
                  note = 1;
                end
      6'd46:    begin
                  trigger = 1;
                  note = 1;
                end
      default:  begin
                  trigger = 0;
                  note = 0;
                end
    endcase
  end

  assign start = trigger & (prescaler == 28'd0);

  // map note number to modulator and carrier frequencies
  // (expressed as delta phase angle per sample, in 8.8-bit
  // fixed-point notation) and modulator and carrier decay
  // rates (expressed as delta phase angle per 256 samples,
  // in 8.4-bit fixed-point notation)
  always @(*) begin
    case (note)
      3'd0:     begin
                  // CSOUND reference bell
                  modFrequ = 16'd921;
                  modDecay = 16'd29;
                  carFrequ = 16'd658;
                  carDecay = 16'd38;
                end
      3'd1:     begin
                  modFrequ = 16'd619;
                  modDecay = 16'd29;
                  carFrequ = 16'd442;
                  carDecay = 16'd38;
                end
      3'd2:     begin
                  modFrequ = 16'd928;
                  modDecay = 16'd29;
                  carFrequ = 16'd663;
                  carDecay = 16'd38;
                end
      3'd3:     begin
                  modFrequ = 16'd1239;
                  modDecay = 16'd29;
                  carFrequ = 16'd885;
                  carDecay = 16'd38;
                end
      3'd4:     begin
                  modFrequ = 16'd1390;
                  modDecay = 16'd29;
                  carFrequ = 16'd993;
                  carDecay = 16'd38;
                end
      3'd5:     begin
                  modFrequ = 16'd1561;
                  modDecay = 16'd29;
                  carFrequ = 16'd1115;
                  carDecay = 16'd38;
                end
      3'd6:     begin
                  modFrequ = 16'd0;
                  modDecay = 16'd0;
                  carFrequ = 16'd0;
                  carDecay = 16'd0;
                end
      3'd7:     begin
                  modFrequ = 16'd0;
                  modDecay = 16'd0;
                  carFrequ = 16'd0;
                  carDecay = 16'd0;
                end
      default:  begin
                  modFrequ = 16'd0;
                  modDecay = 16'd0;
                  carFrequ = 16'd0;
                  carDecay = 16'd0;
                end
    endcase
  end

endmodule
