//
// sine lookup and interpolation
//

module sine(clk, rst, phase, value);
    input clk;
    input rst;
    input [15:0] phase;
    output [15:0] value;

  wire [7:0] addr;
  wire [15:0] sineData;
  wire [15:0] slopeData;
  wire [17:0] factor1;
  wire [17:0] factor2;
  wire [35:0] product;
  wire [15:0] corr;
  reg [7:0] fract;

  assign addr[7:0] = phase[15:8];

  sine_mem sine_mem1(clk, addr[7:0],
                     sineData[15:0],
                     slopeData[15:0]);

  assign factor1 = { 10'b0000000000, fract[7:0] };
  assign factor2 = { slopeData[15],
                     slopeData[15],
                     slopeData[15:0] };
  MULT18X18 interpolator(
    .P(product),    // 36-bit multiplier output
    .A(factor1),    // 18-bit multiplier input
    .B(factor2)     // 18-bit multiplier input
  );
  assign corr = product[23:8] + { 15'b0, product[7] };
  assign value = sineData + corr;

  always @(posedge clk) begin
    if (rst) begin
      fract <= 0;
    end else begin
      fract <= phase[7:0];
    end
  end

endmodule
